program ex10
  implicit none

  integer, parameter :: dp = selected_real_kind(12)
  integer, parameter :: nx = 256, ny = 256
  real(kind=dp), parameter :: dx = 0.01_dp, dy = 0.01_dp

  type :: field
     integer :: nx
     integer :: ny
     real(kind=dp) :: dx
     real(kind=dp) :: dy
     real(kind=dp), dimension(:,:), allocatable :: data
  end type field

  ! Use these types for old and new values
  type(field) :: curr, prev

  call initialize_field(curr, nx, ny, dx, dy)
  call initialize_field(prev, nx, ny, dx, dy)
  call apply_fd_laplacian(curr, prev)
  call output(curr)

contains

  subroutine initialize_field(fld, nx, ny, dx, dy)
    implicit none

    type(field), intent(inout) :: fld
    integer, intent(in) :: nx, ny
    real(kind=dp), intent(in) :: dx, dy
    integer :: allocstat

    !TODO: implement here the initialization of type and boundary values
    !      similarly to the exercise 2

  end subroutine initialize_field

  
  subroutine apply_fd_laplacian(curr, prev)
    implicit none
    type(field), intent(in) :: prev
    type(field) :: curr

    integer :: i, j, lny, uny, lnx, unx

    lny = lbound(prev % data,2)
    uny = ubound(prev % data,2)
    lnx = lbound(prev % data,1)
    unx = ubound(prev % data,1)

    !TODO: compute the Laplacian as in exercise 5

  end subroutine apply_fd_laplacian

  subroutine output(fld)
    use, intrinsic :: ISO_C_BINDING
    implicit none

    type(field), intent(in) :: fld
    integer :: stat

    ! Interface for save_png C-function
    interface
       ! The C-function definition is
       !   int save_png(double *data,
       !                const int nx, const int ny,
       !                const char *fname, const char lang)
       function save_png(data, nx, ny, fname, lang) &
            & bind(C,name="save_png") result(stat)
         use, intrinsic :: ISO_C_BINDING
         implicit none
         real(kind=C_DOUBLE) :: data(*)
         integer(kind=C_INT), value, intent(IN) :: nx, ny
         character(kind=C_CHAR), intent(IN) :: fname(*)
         character(kind=C_CHAR), value, intent(IN) :: lang
         integer(kind=C_INT) :: stat
       end function save_png
    end interface

    stat = save_png(fld%data(1:fld%nx, 1:fld%ny), fld%nx, fld%ny, &
         & 'ex10.png' // C_NULL_CHAR, 'F')
    if (stat == 0) write(*,*) 'Output written to ex7.png'
  end subroutine output

end program ex10
