program ex5b
  implicit none

  integer, parameter :: dp = selected_real_kind(12)
  integer, parameter :: m = 15, n = 15
  real(kind=dp), parameter :: dx=real(0.01,dp), dy=real(0.01,dp)

  real(kind=dp) :: x(0:m+1,0:n+1), y(0:m+1,0:n+1)

  y = real(0, kind=dp)

  call set_initial_values(x)
  call set_initial_values(y)
  call apply_fd_laplacian(x, y, dx, dy)

  if (check_result(x, y, dx, dy)) then
     write (*,*) 'The result seems ok'
  else
     write (*,*) 'The result does not seem ok'
  end if

contains

  subroutine set_initial_values(array)
    implicit none
    real(kind=dp) :: array(:,:)
    integer :: i, lnx, unx, lny, uny

    lnx = lbound(array,1)
    unx = ubound(array,1)
    lny = lbound(array,2)
    uny = ubound(array,2)

    array = real(0,dp)
    ! Left
    do i = lnx, unx
       array(i,lny)=real(20,dp)
    end do
    ! Right
    do i = lnx, unx
       array(i,uny)=real(70,dp)
    end do
    ! Upper
    do i = lny, uny
       array(lnx,i)=real(85,dp)
    end do
    ! Lower
    do i = lny, uny
       array(unx,i)=real(5,dp)
    end do
  end subroutine set_initial_values

  subroutine apply_fd_laplacian(x, y, dx, dy)
    implicit none
    real(kind=dp), intent(in) :: x(:,:)
    real(kind=dp), intent(out) :: y(:,:)
    real(kind=dp), intent(in) :: dx, dy

    real(kind=dp) :: dx2, dy2
    integer :: lnx, unx, lny, uny

    lnx = lbound(x,1)
    unx = ubound(x,1)
    lny = lbound(x,2)
    uny = ubound(x,2)
    dx2 = dx**2
    dy2 = dy**2

    !TODO: Add here the array operations

  end subroutine apply_fd_laplacian

  function check_result(x, y, dx, dy) result(iscorrect)
    implicit none
    real(kind=dp), intent(in) :: x(:,:), y(:,:)
    real(kind=dp), intent(in) :: dx, dy

    real(kind=dp) :: yc(size(x,1),size(x,2))
    integer :: lny, uny, lnx, unx
    real(kind=dp), parameter :: tol = 1e-15
    logical :: iscorrect

    lny = lbound(x,2) + 1
    uny = ubound(x,2) - 1
    lnx = lbound(x,1) + 1
    unx = ubound(x,1) - 1

    yc = real(0,dp)

    yc(lnx:unx,lny:uny) = &
         &    (x(lnx-1:unx-1,lny:uny) - &
         & 2 * x(lnx:unx,lny:uny) + &
         &     x(lnx+1:unx+1,lny:uny))/dx**2 + &
         &    (x(lnx:unx,lny-1:uny-1) - &
         & 2 * x(lnx:unx,lny:uny) + &
         &     x(lnx:unx,lny+1:uny+1))/dy**2

    iscorrect = all(abs(yc(lnx:unx,lny:uny)-y(lnx:unx,lny:uny))<tol)
  end function check_result

end program ex5b
